-----------------------------------------------------------
--|> Core UI: Utils
-----------------------------------------------------------
local _, NS = ...

local gsub = gsub
local string_find = string.find
local WrapTextInColorCode = WrapTextInColorCode
local GetClassColor = GetClassColor
local select = select

--> GET Frame Position
local function GetFramePosition(frame)
  if frame then
    local s = frame:GetScale()
    local left, top = frame:GetLeft() * s, frame:GetTop() * s
    local w, h = frame:GetWidth() * s, frame:GetHeight() * s
    frame:ClearAllPoints()
    frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", left / s, top / s)
    frame:SetWidth(w / s)
    frame:SetHeight(h / s)
    return left, top, w, h, s, "TOPLEFT"
  else
    return nil
  end
end

--> SAVE Core Position
function NS.SaveCoreFramePosition()
  NS.Options.Frames.X, NS.Options.Frames.Y, NS.Options.Frames.Width, _, NS.Options.Frames.Scale, NS.Options.Frames.Point = GetFramePosition(weizPVP_CoreFrame)
  NS.Options.Frames.Height = weizPVP_CoreFrame:GetHeight()
  NS.Options.Frames.List.Height = weizPVP_CoreFrame.ScrollFrame.ListFrame:GetHeight()
end

--> SET Core Position
function NS.SetCoreFramePosition()
  weizPVP_CoreFrame:ClearAllPoints()
  weizPVP_CoreFrame:SetHeight(NS.Options.Frames.Height / NS.Options.Frames.Scale)
  weizPVP_CoreFrame:SetWidth(NS.Options.Frames.Width / NS.Options.Frames.Scale)
  weizPVP_CoreFrame:SetScale(NS.Options.Frames.Scale)
  if (not NS.Options.Frames.X) or (not NS.Options.Frames.Y) then
    weizPVP_CoreFrame:SetPoint("CENTER", UIParent)
    NS.Options.Frames.X, NS.Options.Frames.Y, NS.Options.Frames.Width, _, NS.Options.Frames.Scale, NS.Options.Frames.Point = GetFramePosition(weizPVP_CoreFrame)
  end
  weizPVP_CoreFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", NS.Options.Frames.X / NS.Options.Frames.Scale, NS.Options.Frames.Y / NS.Options.Frames.Scale)
end

--> Format Player Name Only
function NS.FormatPlayerNameOnly(playerName)
  local charName = WrapTextInColorCode(gsub(playerName, "-(.*)", ""), select(4, GetClassColor(NS.PlayerDB[playerName].Class)))
  if not string_find(playerName, NS.PlayerRealm) then
    charName = charName .. "|cFFFF00CC*|r"
  end
  return charName
end

--> Format Player Name And Realm
function NS.FormatPlayerNameAndRealn(playerName)
  if not playerName then
    return "???"
  end
  local charNameAndRealm = WrapTextInColorCode(gsub(playerName, "-(.*)", ""), select(4, GetClassColor(NS.PlayerDB[playerName].Class)))
  charNameAndRealm = charNameAndRealm .. " |cffbbbbbb-|r " .. NS.ColorsLUT["realm"]:WrapTextInColorCode(gsub(playerName, "^(.*-)", ""))
  return charNameAndRealm
end

--> Format Relative Level for Display
function NS.FormatRelativeLevelForDisplay(level)
  if level then
    if level == 0 then --> 0
      return "|cFFFF00CC??|r"
    elseif level < NS.Player.Level - 20 then --> 20+ below
      return "|cFF7cffd1" .. level .. "|r"
    elseif level < NS.Player.Level - 10 then --> 10-20 below
      return "|cFF7cd1ff" .. level .. "|r"
    elseif level < NS.Player.Level then --> 1-9 below
      return "|cFF7cff7f" .. level .. "|r"
    elseif level > NS.Player.Level then --> Higher level
      return "|cFFf7694a" .. level .. "|r"
    else
      return "|cFFffc863" .. level .. "|r"
    end
  end
end

--> Scale Main Window
function NS.ScaleMainWindow()
  weizPVP_CoreFrame:SetScale(NS.Options.Frames.Scale)
  NS.SaveCoreFramePosition()
end
