-----------------------------------------------------------------------
--|> EVENTS
-----------------------------------------------------------------------
local _, NS = ...
local wipe = wipe
local C_PvP_IsWarModeDesired = C_PvP.IsWarModeDesired

NS.PlayerActiveCache = {}

--|> MONITORING
-----------------------------------------------------------------------
--> ENABLE EVENTS
function NS.EnableEvents()
  --: ADDON
  weizPVP:RegisterEvent("ADDON_LOADED", NS.AddonLoadedEvent)
  --: DATA COLLECTION
  weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", NS.NameplateAdded)
  weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", NS.NameplateRemoved)
  weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", NS.CombatLogEvent)
  weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", NS.PlayerTargetEvent)
  weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", NS.PlayerMouseoverEvent)
  weizPVP:RegisterEvent("UNIT_HEALTH", NS.UnitHealthEvent)
  weizPVP:RegisterEvent("UNIT_TARGET", NS.UnitTargetEvent)
  weizPVP:RegisterEvent("UNIT_FLAGS", NS.UnitFlagsChanged)
  --: ZONE CHANGED
  weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", NS.PlayerEnteringBattlegroundEvent)
  weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", NS.PlayerEnteringWorldEvent)
  weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", NS.ZoneChangedNewAreaEvent)
  weizPVP:RegisterEvent("LOADING_SCREEN_ENABLED", NS.LoadingScreenEnabled)
  weizPVP:RegisterEvent("LOADING_SCREEN_DISABLED", NS.LoadingScreenDisabled)
  --: COMBAT/PVP
  weizPVP:RegisterEvent("WAR_MODE_STATUS_UPDATE", NS.WarModeChanged)
  weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", NS.EnteringCombat)
  weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", NS.LeavingCombat)
  --: PLAYER UPDATES
  weizPVP:RegisterEvent("UI_INFO_MESSAGE", NS.UiInfoMessage)
  weizPVP:RegisterEvent("PLAYER_LEVEL_UP", NS.PlayerLevelUp)
  --: PHASE
  weizPVP:RegisterEvent("GARRISON_HIDE_LANDING_PAGE", NS.CheckPhasing)
  --: DISPLAY CHANGES
  weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", NS.UpdateDisplayData)
  weizPVP:RegisterEvent("LOADING_SCREEN_DISABLED", NS.LoadingScreenLeave)
  --: Crosshair
  NS.Crosshair.Enable()
end

--> DISABLE EVENTS
function NS.DisableEvents()
  --: ADDON
  weizPVP:UnregisterEvent("ADDON_LOADED")
  --: DATA COLLECTION
  weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED")
  weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED")
  weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
  weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED")
  weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
  weizPVP:UnregisterEvent("UNIT_HEALTH")
  weizPVP:UnregisterEvent("UNIT_TARGET")
  weizPVP:UnregisterEvent("UNIT_FLAGS")
  --: COMBAT/PVP
  weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED")
  weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED")
  --: PHASE
  weizPVP:UnregisterEvent("GARRISON_HIDE_LANDING_PAGE")
  --: PLAYER UPDATES
  weizPVP:UnregisterEvent("PLAYER_LEVEL_UP")
  --: DISPLAY CHANGES
  weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED")
  weizPVP:UnregisterEvent("LOADING_SCREEN_DISABLED")
  --: Crosshair
  NS.Crosshair.Disable()
end

--|> PLAYER / USER
-----------------------------------------------------------------------
--> [EVENT]: PLAYER LEVELED UP
function NS.PlayerLevelUp()
  NS.Player.Level = UnitLevel("player")
end

--> [EVENT]: DISPLAY_SIZE_CHANGED
function NS.UpdateDisplayData()
  NS.GetDisplayInfo()
  NS.CoreUI.Initialize()
end

--> [EVENT]: UI_INFO_MESSAGE
function NS.UiInfoMessage(_, msgType, _)
  if msgType == 999 then
    NS.WarModeChanged(false)
  elseif msgType == 998 then
    NS.WarModeChanged(true)
  end
end

--> [EVENT]: WAR_MODE_STATUS_UPDATE
function NS.WarModeChanged(enabled)
  if not enabled then
    enabled = C_PvP_IsWarModeDesired()
  end
  if NS.Options.Addon.DisabledWhenWarmodeDisabled and not enabled then
    weizPVP:OnDisable()
  else
    weizPVP:OnEnable()
  end
end

--|> UNITS
-----------------------------------------------------------------------
--> [EVENT]: MOUSEOVER
function NS.PlayerMouseoverEvent()
  NS.GetUnitData("mouseover")
end

--> [EVENT]: TARGET_CHANGED
function NS.PlayerTargetEvent()
  NS.GetUnitData("target")
  NS.Crosshair.NewTarget()
  NS.CoreUI.ChangeTargetIcon()
end

--> [EVENT]: NAMEPLATE_ADDED
function NS.NameplateAdded(_, unit)
  if NS.IsUnitValidForTracking(unit) then
    NS.Crosshair.NameplateAdded(unit)
    NS.GetUnitData(unit)
    NS.CurrentNameplates[UnitGUID(unit)] = true
  end
end

--> [EVENT]: NAMEPLATE_REMOVED
function NS.NameplateRemoved(_, unit)
  if NS.IsUnitValidForTracking(unit) then
    NS.Crosshair.NameplateRemoved(unit)
    if NS.IsUnitValidForTracking(unit) then
      NS.CurrentNameplates[UnitGUID(unit)] = nil
    end
  end
end

--> [EVENT]: HEALTH
function NS.UnitHealthEvent(_, unit)
  if NS.IsUnitValidForTracking(unit) then
    if NS.PlayersOnBars[UnitGUID(unit)] then
      NS.UnitHealthCheck(unit)
    end
  end
end

--> [EVENT]: UNIT TARGET
function NS.UnitTargetEvent(_, unit)
  NS.GetUnitData(unit)
end

--> [EVENT]: UNIT FLAGS CHANGED
local GUID
function NS.UnitFlagsChanged(_, unit)
  GUID = UnitGUID(unit)
  if (not NS.IsUnitValidForTracking(unit)) and NS.PlayerActiveCache[GUID] then
    wipe(NS.PlayerActiveCache[GUID])
    wipe(NS.CurrentList[GUID])
    NS.SortNearbyList()
    if NS.PlayersOnBars[GUID] then
      NS.RefreshCurrentList()
    end
  end
end

--|> ADDON
-----------------------------------------------------------------------
--> [EVENT]: ADDON_LOADED
function NS.AddonLoadedEvent()
  NS.Crosshair.OnLoad()
  weizPVP:UnregisterEvent("ADDON_LOADED")
end

--|> COMBAT
-----------------------------------------------------------------------
--> [EVENT]: ENTERING COMBAT
function NS.EnteringCombat()
  NS.CoreUI.CombatStart()
end

--> [EVENT]: LEAVING COMBAT
function NS.LeavingCombat()
  NS.ManageListTimeouts()
  NS.CoreUI.CombatEnd()
end
