-----------------------------------------------------------------------
--|> COMBAT LOG
-----------------------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
--> UpValues
local bit_band = bit.band
local string_match = string.match
local select = select
local strsub = strsub
local CombatLogGetCurrentEventInfo = CombatLogGetCurrentEventInfo
local GetTime = GetTime
local GetPlayerInfoByGUID = GetPlayerInfoByGUID
local GetSpecializationInfoByID = GetSpecializationInfoByID
local CL_HOSTILE = COMBATLOG_OBJECT_REACTION_HOSTILE
-- local CL_PLAYER = COMBATLOG_OBJECT_TYPE_PLAYER
local GetSpellInfo = GetSpellInfo

--|> Auras Constants
-----------------------------------------------------------------------
local holyPriest = {
  [27827] = GetSpellInfo(27827),
  [20711] = GetSpellInfo(20711),
  [126094] = GetSpellInfo(126094)
}
local stealthIDs = {
  [1784] = true, -- Stealth
  [6770] = true, -- Sap
  [6409] = true, -- Cheap Shot
  [115834] = true, -- Shroud of Concealment
  [114018] = true, -- Shroud of Concealment
  [11327] = true, -- Vanish
  [5215] = true, -- prowl
  [66] = true, -- Invisibility
  [156136] = true, -- stealth-field
  [113862] = true, -- Greater Invisibility
  [250960] = true -- potion of concealment
}
local stealthNames = {
  ["Stealth"] = true,
  ["Prowl"] = true,
  ["Invisibility"] = true
}

--|> Locals
-----------------------------------------------------------------------
--> Static Role Assignment
local function RoleAssign(class)
  if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
    return "DAMAGER"
  end
  return nil
end

--> Get Role from Spec
local function GetSpecRole(specId)
  if specId then
    return select(5, GetSpecializationInfoByID(specId))
  end
  return nil
end

--> Role Check
local function ClRoleCheck(spellId, class)
  local role = RoleAssign(class)
  if role ~= nil then
    return role
  end
  local specId = NS.GetSpecIdFromSpellId(spellId)
  if not specId then
    return nil
  end
  role = GetSpecRole(specId)
  return role
end

--> Is Valid Player?
local function IsValidPlayer(flags, guid)
  if (not flags) or (not guid) then
    return false
  end
  if not string_match(guid, "Player") then
    return false
  end
  if ((bit_band(flags, CL_HOSTILE) == CL_HOSTILE) or NS.Options.DEBUG) then
    -- if ((bit_band(flags, CL_HOSTILE) == CL_HOSTILE) or NS.Options.DEBUG) and bit_band(flags, CL_PLAYER) == CL_PLAYER then
    return true
  end
  return false
end

--> CL: CREATE NEW PLAYER ENTRY IN DB
local function AddNewPlayerToDatabase(event, srcName, srcGUID, spellId)
  if srcName and srcGUID then
    NS.PlayerDB[srcName] = {}
    NS.PlayerDB[srcName].Name = srcName
    NS.PlayerDB[srcName].Timestamp = GetTime()
    NS.PlayerDB[srcName].Estimated = true

    local abilityType = strsub(event, 1, 5)
    local levelNewPlayer = 0
    if abilityType == "SWING" or abilityType == "SPELL" or abilityType == "RANGE" then
      levelNewPlayer = NS.GetLevelFromSpellId(spellId) or 0
    end
    NS.PlayerDB[srcName].Level = levelNewPlayer
    _, class = GetPlayerInfoByGUID(srcGUID)
    NS.PlayerDB[srcName].Class = class
    NS.PlayerDB[srcName].Role = ClRoleCheck(spellId, class) or nil
  end
end

--> CL: DEATH CHECK
local function CLDeath(fEvent, fGUID, fName)
  if NS.PlayerActiveCache[fGUID] and fEvent == "UNIT_DIED" then
    NS.PlayerActiveCache[fGUID].Dead = true
    NS.UpdatePlayerActiveCache(fName, nil, true, nil, fGUID)
    return true
  end
  return false --: return false on no death seen
end

--> Aura Check
local icon
local stealthFound
local function AuraCheck(spellId, spellName, srcGUID, srcName)
  --: Stealth check
  stealthFound = false
  if stealthNames[spellName] then
    stealthFound = true
  elseif stealthIDs[spellId] then
    stealthFound = true
  end
  if stealthFound  then
    if NS.Options.StealthAlert.Enabled  then
      _, _, icon = GetSpellInfo(spellId)
      NS.ShowAlertMessage(spellName, NS.FormatPlayerNameAndRealn(srcName), icon)
    end
    NS.UpdatePlayerActiveCache(srcName, true, false, nil, srcGUID)
    return true
  end
  --: Redemption check
  if holyPriest[spellId] then
    NS.UpdatePlayerActiveCache(srcName, nil, true, "HEALER", srcGUID)
    return true --: Return after updating player cache
  end
  return false
end

local function FormatCLogName(srcName)
  if not srcName then
    return nil
  end
  if not string_match(srcName, "-") then
    srcName = srcName .. "-" .. NS.PlayerRealm
  end
  return srcName
end

--|> Process Source
local function ParseSource(eventData)
  if IsValidPlayer(eventData[6], eventData[4]) then
    if string_match(eventData[5], "Unknown") then
      return
    end
    local event = eventData[2]
    local srcGUID = eventData[4]
    local srcName = FormatCLogName(eventData[5])
    local spellId = eventData[12]
    local spellName = eventData[13]
    if not srcName then
      return
    end
    --: Check if unit is not in DB yet
    if not NS.PlayerDB[srcName] then
      AddNewPlayerToDatabase(event, srcName, srcGUID, spellId)
      NS.UpdatePlayerActiveCache(srcName, nil, false, nil, srcGUID) --(name, stealth, dead, role, GUID)
    end
    --: Level Check
    if NS.PlayerDB[srcName].Estimated  then
      local abilityType = strsub(event, 1, 5)
      if abilityType == "SWING" or abilityType == "SPELL" or abilityType == "RANGE" then
        local newLevel = NS.GetLevelFromSpellId(spellId) or 0
        NS.PlayerDB[srcName].Level = (newLevel > NS.PlayerDB[srcName].Level) and newLevel or NS.PlayerDB[srcName].Level
      end
    end
    --: Check role
    -- if NS.PlayersOnBars[srcGUID] and NS.PlayerActiveCache[srcGUID] and NS.PlayerActiveCache[srcGUID].RoleFound then
    if NS.PlayersOnBars[srcGUID] and not NS.PlayerActiveCache[srcGUID].RoleFound then
      local srcRole = ClRoleCheck(spellId, NS.PlayerDB[srcName].Class)
      if srcRole ~= nil then
        if NS.PlayerDB[srcName].Role ~= srcRole then
          NS.PlayerDB[srcName].Role = srcRole
        end
        NS.PlayerActiveCache[srcGUID].RoleFound = true
      end
    end
    --: AuraCheck
    if event == "SPELL_AURA_APPLIED" then
      if AuraCheck(spellId, spellName, srcGUID, srcName)  then
        return
      end
    end
    NS.UpdatePlayerActiveCache(srcName, nil, false, nil, srcGUID)
  end
end

local function ParseDestination(eventData)
  if IsValidPlayer(eventData[10], eventData[8]) then
    if string_match(eventData[9], "Unknown") then
      return
    end
    CLDeath(eventData[2], eventData[8], FormatCLogName(eventData[9]))
  end
end

--|> NS Functions
--> [EVENT]: COMBAT LOG
-- local eventData
function NS.CombatLogEvent()
  local eventData = {CombatLogGetCurrentEventInfo()} or nil
  if not eventData then
    return
  end
  ParseSource(eventData)
  ParseDestination(eventData)
end
