-----------------------------------------------------------------------
--|> ZONE DETECTION
-----------------------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
--> UpValues
local select = select
local wipe = wipe
local C_Timer_After = C_Timer.After
local IsInInstance = IsInInstance
local GetZoneText = GetZoneText

--|> ZONES AND INSTANCES
NS.Zone = {}
NS.Zone.instance = ""
NS.Zone.pvpType = ""
NS.Zone.InInstance = nil

--> Check for Valid (PVP) Instance
local function CheckValidInstance()
  if NS.Zone.instance == "pvp" and NS.Options.Addon.EnabledInBattlegrounds then
    return true
  end
  if NS.Zone.instance == "arena" and NS.Options.Addon.EnabledInArena then
    return true
  end
  return false
end

--> Check for Valid World Zone
local function CheckValidWorldZone()
  --> Checking for Valid WPVP Zone
  if NS.Zone.instance ~= "none" then
    return false
  end
  if NS.Options.Addon.DisabledInSanctuary and NS.Zone.pvpType == "sanctuary" then
    return false
  else
    return true
  end
end

--> Get PVP Zone
local gettingZone = nil
local inInstance = nil
local function GetZoneType()
  if GetZoneText() == "" then
    C_Timer_After(0.25, GetZoneType)
    return
  end
  NS.Zone.pvpType = select(1, GetZonePVPInfo())
  NS.Zone.InInstance, NS.Zone.instance = IsInInstance()
  if NS.Zone.InInstance ~= inInstance then
    NS.ClearListData()
    inInstance = NS.Zone.InInstance
  end
  if CheckValidInstance() then
    weizPVP:OnEnable()
  elseif CheckValidWorldZone() then
    NS.WarModeChanged()
  else
    weizPVP:OnDisable()
  end
  gettingZone = nil
end

function NS.GetPVPZone()
  if gettingZone then
    return
  end
  gettingZone = true
  GetZoneType()
end

--> [EVENT]: Entering Battleground Instance
function NS.PlayerEnteringBattlegroundEvent()
  NS.GetPVPZone()
  wipe(NS.CurrentNameplates)
end

--> [EVENT]: ZoneChangedNewAreaEvent
function NS.ZoneChangedNewAreaEvent()
  NS.GetPVPZone()
end

--> [EVENT]: Loading Screen Leave
function NS.LoadingScreenLeave()
  -- NS.CoreUI.Initialize()
end

--> [EVENT]: Player Entering World
function NS.PlayerEnteringWorldEvent()
  if NS.addonInitializing then -- check if this is the first 'entering world' run since init was ran
    NS.PrintAddonMessage(NS.Constants.AddonString .. " |cffcccccc" .. GetAddOnMetadata("weizPVP", "Version") .. "|r : |cff37ff37Initialized!|r")
    NS.addonInitializing = nil
  end
  --> ENABLES
  NS.EnableLDB()
  wipe(NS.CurrentNameplates)
  NS.Crosshair.NewTarget() -- check for target
  if NS.databaseReset then -- database update check for pre-1.9.0
    C_Timer_After(
      1,
      function()
        NS.databaseReset = nil
        StaticPopup_Show("WEIZPVP_UPGRADE_DB_RESET")
        NS.PrintAddonMessage("|cffFFA200Saved options and data have been reset!|r")
        NS.PrintAddonMessage(
          "The database reset was required to resolve potential issues with the new KOS features and player tracking, especially for players that play on multiple realms."
        )
        NS.PrintAddonMessage("Database updates in the future will migrate all your data and settings; No more resets will be needed.")
      end
    )
  end
  -- C_Timer_After(1, NS.CheckPhasing)
  NS.CoreUI.Initialize()
  NS.SetWindowSettings()
  NS.GetPVPZone()
end

--> [EVENT]: Loading Scren Enabled
function NS.LoadingScreenEnabled()
  weizPVP.loadingScreen = true
end

--> [EVENT]: Loading Scren Disabled
function NS.LoadingScreenDisabled()
  weizPVP.loadingScreen = nil
end
