-----------------------------------------------------------
--|> UNIT PROCESSING
--: Process Unit Data
-----------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
--> UpValues
local gsub = gsub
local select = select

--> Unescape
local escapes = {
  ["|c%x%x%x%x%x%x%x%x"] = "", -- color start
  ["|r"] = "", -- color end
  ["|H.-|h(.-)|h"] = "%1", -- links
  ["|T.-|t"] = "", -- textures
  ["{.-}"] = "" -- raid target icons
}

function NS.Unescape(str)
  for k, v in pairs(escapes) do
    str = gsub(str, k, v)
  end
  return str
end

--> GET COLOR VALUE FROM GRADIENT
--: value = 0-1 value
--: ... = color values
local rGrad, gGrad, bGrad
local numGrad
local segment, relperc
local r1, g1, b1, r2, g2, b2 = 1, 0, 0, 1, 0, 0
function NS.GetColorValueFromGradient(value, ...)
  if value >= 1 then
    rGrad, gGrad, bGrad = select(select("#", ...) - 2, ...)
    NS.RGB2Hex(abs(rGrad * 255), abs(gGrad * 255), abs(bGrad * 255))
  elseif value <= 0 then
    rGrad, gGrad, bGrad = ...
    NS.RGB2Hex(abs(rGrad * 255), abs(gGrad * 255), abs(bGrad * 255))
  end
  numGrad = select("#", ...) / 3
  segment, relperc = math.modf(value * (numGrad - 1))
  r1, g1, b1, r2, g2, b2 = select((segment * 3) + 1, ...)
  rGrad = r1 + (r2 - r1) * relperc
  gGrad = g1 + (g2 - g1) * relperc
  bGrad = b1 + (b2 - b1) * relperc
  return NS.RGB2Hex(abs(rGrad * 255), abs(gGrad * 255), abs(bGrad * 255))
end

--> RGB TO HSV
function NS.RGB2Hex(r, g, b)
  r = r <= 255 and r >= 0 and r or 0
  g = g <= 255 and g >= 0 and g or 0
  b = b <= 255 and b >= 0 and b or 0
  return string.format("%02x%02x%02x", r, g, b)
end

--> RESCALE FRAME
local function RescaleRegionSize(region)
  if not region then
    return
  end
  if not region.rescaled then
    region:SetWidth(NS.ScalePixelsToUi(region:GetWidth()))
    region:SetHeight(NS.ScalePixelsToUi(region:GetHeight()))
    region.rescaled = true
  end
end

function NS.ResizeFrameToUiScale(frame)
  if (not frame) or (not frame.rescaled) then
    return
  end
  RescaleRegionSize(frame)
  return
end

--> GET UI MULTIPLIER
function NS.GetUiMultiplier()
  return ConvertPixelsToUI(1, UIParent:GetEffectiveScale())
end

--> SCALE PIXELS TO WIDTH
function NS.ScalePixelsToUi(pixels)
  if not pixels then
    return nil
  end
  return (NS.GetUiMultiplier() * pixels)
end
